from .utils import hash_code, get_random_color, render

ELEMENTS = 64
SIZE = 80


def generate_colors(name, colors):
    num_from_name = hash_code(name)
    return [
        get_random_color(num_from_name % (i + 1), colors, len(colors))
        for i in range(ELEMENTS)
    ]


def pixel(name, *, colors, size, title, square):
    pixel_colors = generate_colors(name, colors)
    return render(
        "pixel.svg",
        {
            "name": name,
            "pixel_colors": pixel_colors,
            "SIZE": SIZE,
            "size": size,
            "title": title,
            "square": square,
        },
    )



import pytest

from boringavatars import avatar


@pytest.mark.parametrize(
    "variant", ["beam", "marble", "pixel", "sunset", "bauhaus", "ring"]
)
def test_avatar(variant):
    avatar("foobar", variant=variant)



from .utils import hash_code, get_random_color, get_unit, render

ELEMENTS = 3
SIZE = 80


def generate_colors(name, colors):
    num_from_name = hash_code(name)

    return [
        {
            "color": get_random_color(num_from_name + i, colors, len(colors)),
            "translate_x": get_unit(num_from_name * (i + 1), SIZE / 10, 1),
            "translate_y": get_unit(num_from_name * (i + 1), SIZE / 10, 2),
            "scale": 1.2 + get_unit(num_from_name * (i + 1), SIZE / 20) / 10,
            "rotate": get_unit(num_from_name * (i + 1), 360, 1),
        }
        for i in range(ELEMENTS)
    ]


def marble(name, *, colors, size, title, square):
    properties = generate_colors(name, colors)
    return render(
        "marble.svg",
        {
            "name": name,
            "properties": properties,
            "SIZE": SIZE,
            "size": size,
            "title": title,
            "square": square,
        },
    )



from .utils import hash_code, get_random_color, get_unit, render

SIZE = 90
COLORS = 5


def generate_colors(name, colors):
    num_from_name = hash_code(name)
    colors_shuffle = [
        get_random_color(num_from_name + i, colors, len(colors)) for i in range(COLORS)
    ]
    return [
        colors_shuffle[0],
        colors_shuffle[1],
        colors_shuffle[1],
        colors_shuffle[2],
        colors_shuffle[2],
        colors_shuffle[3],
        colors_shuffle[3],
        colors_shuffle[0],
        colors_shuffle[4],
    ]


def ring(name, *, colors, size, title, square):
    ring_colors = generate_colors(name, colors)
    return render(
        "ring.svg",
        {
            "name": name,
            "ring_colors": ring_colors,
            "SIZE": SIZE,
            "size": size,
            "title": title,
            "square": square,
        },
    )



import hashlib
import math
import os.path

from jinja2 import Environment, FileSystemLoader, select_autoescape


def hash_code(name):
    return int.from_bytes(hashlib.sha1(name.encode("utf-8")).digest(), byteorder="big")


def get_digit(number, ntn):
    return math.floor((number // int(math.pow(10, ntn))) % 10)


def get_boolean(number, ntn):
    return not ((get_digit(number, ntn)) % 2)


def get_angle(x, y):
    return math.atan2(y, x) * 180 / math.pi


def get_unit(number, range, index=None):
    value = number % range

    if index and ((get_digit(number, index) % 2) == 0):
        return -value
    else:
        return value


def get_random_color(number, colors, range):
    return colors[(number) % range]


def get_contrast(hexcolor):
    # If a leading # is provided, remove it
    if hexcolor[0] == "#":
        hexcolor = hexcolor[1:]

    # Convert to RGB value
    r = int(hexcolor[0:2], 16)
    g = int(hexcolor[2:4], 16)
    b = int(hexcolor[4:6], 16)

    # Get YIQ ratio
    yiq = ((r * 299) + (g * 587) + (b * 114)) / 1000

    # Check contrast
    return "#000000" if yiq >= 128 else "#FFFFFF"


template_path = os.path.join(os.path.dirname(__file__), "templates")
env = Environment(
    loader=FileSystemLoader(template_path), autoescape=select_autoescape()
)


def render(template_name, context):
    return env.get_template(template_name).render(context)



import hashlib

from .utils import hash_code, get_random_color, render

ELEMENTS = 4
SIZE = 80


def generate_colors(name, colors):
    num_from_name = hash_code(name)
    return [
        get_random_color(num_from_name + i, colors, len(colors))
        for i in range(ELEMENTS)
    ]


def sunset(name, *, colors, size, title, square):
    sunset_colors = generate_colors(name, colors)
    name = hashlib.sha1(name.encode("utf-8")).hexdigest()
    return render(
        "sunset.svg",
        {
            "name": name,
            "sunset_colors": sunset_colors,
            "SIZE": SIZE,
            "size": size,
            "title": title,
            "square": square,
        },
    )



from .utils import (
    hash_code,
    get_boolean,
    get_contrast,
    get_random_color,
    get_unit,
    render,
)

SIZE = 36


def generate_data(name, colors):
    num_from_name = hash_code(name)
    wrapper_color = get_random_color(num_from_name, colors, len(colors))
    pre_translate_x = get_unit(num_from_name, 10, 1)
    wrapper_translate_x = (
        pre_translate_x + SIZE / 9 if pre_translate_x < 5 else pre_translate_x
    )
    pre_translate_y = get_unit(num_from_name, 10, 2)
    wrapper_translate_y = (
        pre_translate_y + SIZE / 9 if pre_translate_y < 5 else pre_translate_y
    )

    return {
        "wrapper_color": wrapper_color,
        "face_color": get_contrast(wrapper_color),
        "background_color": get_random_color(num_from_name + 13, colors, len(colors)),
        "wrapper_translate_x": wrapper_translate_x,
        "wrapper_translate_y": wrapper_translate_y,
        "wrapper_rotate": get_unit(num_from_name, 360),
        "wrapper_scale": 1 + get_unit(num_from_name, SIZE / 12) / 10,
        "is_mouth_open": get_boolean(num_from_name, 2),
        "is_circle": get_boolean(num_from_name, 1),
        "eye_spread": get_unit(num_from_name, 5),
        "mouth_spread": get_unit(num_from_name, 3),
        "face_rotate": get_unit(num_from_name, 10, 3),
        "face_translate_x": wrapper_translate_x / 2
        if wrapper_translate_x > SIZE / 6
        else get_unit(num_from_name, 8, 1),
        "face_translate_y": wrapper_translate_y / 2
        if wrapper_translate_y > SIZE / 6
        else get_unit(num_from_name, 7, 2),
    }


def beam(name, *, colors, size, title, square):
    context = generate_data(name, colors)
    context["name"] = name
    context["SIZE"] = SIZE
    context["size"] = size
    context["title"] = title
    context["square"] = square
    return render("beam.svg", context)



from .utils import hash_code, get_boolean, get_random_color, get_unit, render

ELEMENTS = 4
SIZE = 80


def generate_colors(name, colors):
    num_from_name = hash_code(name)

    return [
        {
            "color": get_random_color(num_from_name + i, colors, len(colors)),
            "translate_x": get_unit(num_from_name * (i + 1), SIZE / 2 - (i + 17), 1),
            "translate_y": get_unit(num_from_name * (i + 1), SIZE / 10 - (i + 17), 2),
            "rotate": get_unit(num_from_name * (i + 1), 360),
            "is_square": get_boolean(num_from_name, 2),
        }
        for i in range(ELEMENTS)
    ]


def bauhaus(name, *, colors, size, title, square):
    properties = generate_colors(name, colors)
    return render(
        "bauhaus.svg",
        {
            "name": name,
            "properties": properties,
            "SIZE": SIZE,
            "size": size,
            "title": title,
            "square": square,
        },
    )



from .bauhaus import bauhaus
from .beam import beam
from .marble import marble
from .pixel import pixel
from .ring import ring
from .sunset import sunset

__version__ = "1.0.1"

__all__ = ["avatar"]

DEFAULT_COLORS = ["FFAD08", "EDD75A", "73B06F", "0C8F8F", "405059"]

VARIANTS = {
    "beam": beam,
    "marble": marble,
    "pixel": pixel,
    "sunset": sunset,
    "bauhaus": bauhaus,
    "ring": ring,
}


def avatar(
    name: str,
    *,
    colors: list[str] = DEFAULT_COLORS,
    variant: str = "marble",
    title: bool = False,
    size: int = 40,
    square: bool = False,
) -> str:
    """Generate an avatar SVG string with the given parameters."""
    if variant not in VARIANTS:
        raise ValueError(f"unrecognized variant: {variant}")

    fn = VARIANTS[variant]
    colors = [c.lstrip("#") for c in colors]
    return fn(name, colors=colors, size=size, title=title, square=square)
